@@ -33,6 +33,14 @@
 
 #include "libxsmm_macros.h"
 
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
+#include <stddef.h>
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
+
 
 /** Flag enumeration which can be binary ORed. */
 typedef enum libxsmm_gemm_flags {
@@ -46,6 +54,14 @@ typedef enum libxsmm_gemm_flags {
   LIBXSMM_GEMM_FLAG_ALIGN_C = 8
 } libxsmm_gemm_flags;
 
+/** Extended flag set complementing libxsmm_gemm_flags. */
+typedef enum libxsmm_gemm_precision {
+  /** Not an actual flag; just provided for symmetry. */
+  LIBXSMM_GEMM_FLAG_F64PREC = 0,
+  /** Single-precision (sgemm rather than dgemm). */
+  LIBXSMM_GEMM_FLAG_F32PREC = 16
+} libxsmm_gemm_precision;
+
 /** Enumeration of the available prefetch strategies. */
 typedef enum libxsmm_gemm_prefetch_type {
   /** Automatically select strategy (frontend). */
@@ -122,25 +138,25 @@ typedef enum libxsmm_convolution_prefetch_type {
 } libxsmm_convolution_prefetch_type;
 
 
-typedef enum libxsmm_dnn_conv_format{
+typedef enum libxsmm_dnn_tensor_format{
   /* use LIBXSMM internal format, we need to copy data into that */
-  LIBXSMM_DNN_CONV_FORMAT_LIBXSMM = 1,
+  LIBXSMM_DNN_TENSOR_FORMAT_LIBXSMM = 1,
   /* use NHWC format internally, this allows no-copy operations */
-  LIBXSMM_DNN_CONV_FORMAT_NHWC = 2,
+  LIBXSMM_DNN_TENSOR_FORMAT_NHWC = 2,
   /* use NCHW format internally, this will include shadow copies, not preferred */
-  LIBXSMM_DNN_CONV_FORMAT_NCHW = 4,
+  LIBXSMM_DNN_TENSOR_FORMAT_NCHW = 4,
   /* use RSCK format internally, this allows no-copy operations  */
-  LIBXSMM_DNN_CONV_FORMAT_RSCK = 8,
+  LIBXSMM_DNN_TENSOR_FORMAT_RSCK = 8,
   /* use KCRS format internally, this will include shadow copies, not preferred */
-  LIBXSMM_DNN_CONV_FORMAT_KCRS = 16,
+  LIBXSMM_DNN_TENSOR_FORMAT_KCRS = 16,
   /* use ptr copy when copying in -> no copy takes place, this is just an additional option */
-  LIBXSMM_DNN_CONV_FORMAT_PTR = 32,
+  LIBXSMM_DNN_TENSOR_FORMAT_PTR = 32,
   /* now some combinded types */
-  LIBXSMM_DNN_CONV_FORMAT_NHWC_PTR = LIBXSMM_DNN_CONV_FORMAT_NHWC | LIBXSMM_DNN_CONV_FORMAT_PTR,
-  LIBXSMM_DNN_CONV_FORMAT_RSCK_PTR = LIBXSMM_DNN_CONV_FORMAT_RSCK | LIBXSMM_DNN_CONV_FORMAT_PTR,
-  LIBXSMM_DNN_CONV_FORMAT_NHWC_RSCK = LIBXSMM_DNN_CONV_FORMAT_NHWC | LIBXSMM_DNN_CONV_FORMAT_RSCK,
-  LIBXSMM_DNN_CONV_FORMAT_LIBXSMM_PTR = LIBXSMM_DNN_CONV_FORMAT_LIBXSMM | LIBXSMM_DNN_CONV_FORMAT_PTR
-} libxsmm_dnn_conv_format;
+  LIBXSMM_DNN_TENSOR_FORMAT_NHWC_PTR = LIBXSMM_DNN_TENSOR_FORMAT_NHWC | LIBXSMM_DNN_TENSOR_FORMAT_PTR,
+  LIBXSMM_DNN_TENSOR_FORMAT_RSCK_PTR = LIBXSMM_DNN_TENSOR_FORMAT_RSCK | LIBXSMM_DNN_TENSOR_FORMAT_PTR,
+  LIBXSMM_DNN_TENSOR_FORMAT_NHWC_RSCK = LIBXSMM_DNN_TENSOR_FORMAT_NHWC | LIBXSMM_DNN_TENSOR_FORMAT_RSCK,
+  LIBXSMM_DNN_TENSOR_FORMAT_LIBXSMM_PTR = LIBXSMM_DNN_TENSOR_FORMAT_LIBXSMM | LIBXSMM_DNN_TENSOR_FORMAT_PTR
+} libxsmm_dnn_tensor_format;
 
 /** Denotes the element/pixel type of an image/channel. */
 typedef enum libxsmm_dnn_datatype {
@@ -156,7 +172,11 @@ typedef enum libxsmm_dnn_conv_option {
   /* activations are stored unsigned */
   LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED = 1,
   /* reduce filters externally to op */
-  LIBXSMM_DNN_CONV_OPTION_WU_EXT_FILTER_REDUCE = 2
+  LIBXSMM_DNN_CONV_OPTION_WU_EXT_FILTER_REDUCE = 2,
+  /* use 16 bit accumulate instead of 32 bit accumulate for fix point */
+  LIBXSMM_DNN_CONV_OPTION_16BIT_ACC = 4,
+  /* compound types */
+  LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED_16BIT_ACC = LIBXSMM_DNN_CONV_OPTION_ACTIVATION_UNSIGNED | LIBXSMM_DNN_CONV_OPTION_16BIT_ACC
 } libxsmm_dnn_conv_option;
 
 /** Structure storing the convolution argument description. */
@@ -178,10 +198,10 @@ typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_forward_descriptor {
   unsigned int stride_h;                        /* this we use for offsets in the input */
   unsigned int stride_w;                        /* this we use for offsets in the input */
   unsigned int fm_lp_block;                    /* additional blocking for low precision datatypes of ifm */
-  libxsmm_dnn_conv_format format;
+  libxsmm_dnn_tensor_format format;
   libxsmm_dnn_conv_option option;
-  libxsmm_dnn_datatype datatype_in;
-  libxsmm_dnn_datatype datatype_out;
+  libxsmm_dnn_datatype datatype;
+  libxsmm_dnn_datatype datatype_itm;
   libxsmm_convolution_prefetch_type prefetch;   /* prefetch type, can be ORed vales of libxsmm_convolution_prefetch_type */
 } libxsmm_convolution_forward_descriptor;
 
@@ -210,10 +230,10 @@ typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_backward_descriptor {
 
   unsigned int prefetch_output_ahead;           /* prefetch all outputs of kj when you jump from non-peeled to peeled */
 
-  libxsmm_dnn_conv_format format;
+  libxsmm_dnn_tensor_format format;
   libxsmm_dnn_conv_option option;
-  libxsmm_dnn_datatype datatype_in;
-  libxsmm_dnn_datatype datatype_out;
+  libxsmm_dnn_datatype datatype;
+  libxsmm_dnn_datatype datatype_itm;
   libxsmm_convolution_prefetch_type prefetch;   /* prefetch type, can be ORed vales of libxsmm_convolution_prefetch_type */
 } libxsmm_convolution_backward_descriptor;
 
@@ -242,10 +262,10 @@ typedef struct LIBXSMM_MAY_ALIAS libxsmm_convolution_weight_update_descriptor {
   unsigned int ofw_unroll;                      /* this we use to unroll ofw loop */
 
   unsigned int transpose_ofw_ifm;               /* transpose ofw and ifm */
-  libxsmm_dnn_conv_format format;
+  libxsmm_dnn_tensor_format format;
   libxsmm_dnn_conv_option option;
-  libxsmm_dnn_datatype datatype_in;
-  libxsmm_dnn_datatype datatype_out;
+  libxsmm_dnn_datatype datatype;
+  libxsmm_dnn_datatype datatype_itm;
   libxsmm_convolution_prefetch_type prefetch;   /* prefetch type, can be ORed vales of libxsmm_convolution_prefetch_type */
 } libxsmm_convolution_weight_update_descriptor;
 
@@ -256,5 +276,8 @@ typedef LIBXSMM_RETARGETABLE void (*libxsmm_dmmfunction)(const double* a, const
 /** Function type which is either libxsmm_smmfunction or libxsmm_dmmfunction (weak-typed). */
 typedef union LIBXSMM_RETARGETABLE libxsmm_xmmfunction { libxsmm_smmfunction smm; libxsmm_dmmfunction dmm; } libxsmm_xmmfunction;
 
+/** Structure to receive information about the code registry status (libxsmm_get_registry_info). */
+typedef struct LIBXSMM_RETARGETABLE libxsmm_registry_info { size_t capacity, size, nbytes, nstatic, ncache; } libxsmm_registry_info;
+
 #endif /*LIBXSMM_TYPEDEFS_H*/
 
