@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2014-2016, Intel Corporation                                **
+** Copyright (c) 2014-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -41,6 +41,16 @@
 #include <mkl.h>
 #endif
 
+#ifndef __INTEL_COMPILER
+/* based on: http://www.mcs.anl.gov/~kazutomo/rdtsc.html */
+static __inline__ unsigned long long _rdtsc(void)
+{
+    unsigned hi, lo;
+    __asm__ __volatile__ ("rdtsc" : "=a"(lo), "=d"(hi));
+    return ( (unsigned long long)lo)|( ((unsigned long long)hi)<<32 );
+}
+#endif
+
 /*#define STREAM_A_B*/
 #ifdef STREAM_A_B
 #define STREAM_A_B_SIZE 1000
@@ -84,7 +94,7 @@ static double sec(struct timeval start, struct timeval end) {
   return ((double)(((end.tv_sec * 1000000 + end.tv_usec) - (start.tv_sec * 1000000 + start.tv_usec)))) / 1.0e6;
 }
 
-void run_test() {
+void run_test(void) {
   /* allocate */
 #ifdef STREAM_A_B
   REALTYPE* l_a = (REALTYPE*)_mm_malloc(MY_LDA * MY_K * sizeof(REALTYPE) * STREAM_A_B_SIZE, 64);
@@ -168,10 +178,10 @@ void run_test() {
     int l_ldb = MY_LDB;
     int l_ldc = MY_LDC;
     if (sizeof(REALTYPE) == sizeof(double)) {
-      double l_one = 1.0;    
-      dgemm(&l_trans, &l_trans, &l_M, &l_N, &l_K, &l_one, (double*)l_a, &l_lda, (double*)l_b, &l_ldb, &l_one, (double*)l_c_gold, &l_ldc); 
+      double l_one = 1.0;
+      dgemm(&l_trans, &l_trans, &l_M, &l_N, &l_K, &l_one, (double*)l_a, &l_lda, (double*)l_b, &l_ldb, &l_one, (double*)l_c_gold, &l_ldc);
     } else {
-      float l_one = 1.0f;    
+      float l_one = 1.0f;
       sgemm(&l_trans, &l_trans, &l_M, &l_N, &l_K, &l_one, (float*)l_a, &l_lda, (float*)l_b, &l_ldb, &l_one, (float*)l_c_gold, &l_ldc);
     }
   }
@@ -183,7 +193,7 @@ void run_test() {
       l_c_gold[(l_j * MY_LDC) + l_i] = (REALTYPE)0.0;
     }
   }
-#endif   
+#endif
 
   /* C routine */
   gettimeofday(&l_start, NULL);
@@ -232,14 +242,14 @@ void run_test() {
 #else
         REALTYPE* l_p_a = l_a;
         REALTYPE* l_p_b = l_b;
-#endif 
+#endif
         dgemm(&l_trans, &l_trans, &l_M, &l_N, &l_K, &l_one, (double*)l_p_a, &l_lda, (double*)l_p_b, &l_ldb, &l_one, (double*)l_c_gold, &l_ldc);
 #ifdef STREAM_A_B
       }
 #endif
-    } 
+    }
   } else {
-    float l_one = 1.0f;   
+    float l_one = 1.0f;
     for ( l_t = 0; l_t < REPS; l_t++  ) {
 #ifdef STREAM_A_B
       REALTYPE* l_p_a = l_a - (MY_K * MY_LDA);
@@ -250,7 +260,7 @@ void run_test() {
 #else
         REALTYPE* l_p_a = l_a;
         REALTYPE* l_p_b = l_b;
-#endif  
+#endif
         sgemm(&l_trans, &l_trans, &l_M, &l_N, &l_K, &l_one, (float*)l_p_a, &l_lda, (float*)l_p_b, &l_ldb, &l_one, (float*)l_c_gold, &l_ldc);
 #ifdef STREAM_A_B
       }
@@ -259,7 +269,7 @@ void run_test() {
   }
 #endif
   gettimeofday(&l_end, NULL);
-  
+
   l_total = sec(l_start, l_end);
 #ifndef __USE_MKL
   printf("%fs for C\n", l_total);
@@ -300,7 +310,7 @@ void run_test() {
     }
 #endif
   }
-  size_t l_cyc_end = _rdtsc();  
+  size_t l_cyc_end = _rdtsc();
   gettimeofday(&l_end, NULL);
   l_total = sec(l_start, l_end);
 
