@@ -1,6 +1,6 @@
-#! /usr/bin/env python
+#!/usr/bin/env python
 ###############################################################################
-## Copyright (c) 2014-2016, Intel Corporation                                ##
+## Copyright (c) 2014-2017, Intel Corporation                                ##
 ## All rights reserved.                                                      ##
 ##                                                                           ##
 ## Redistribution and use in source and binary forms, with or without        ##
@@ -31,7 +31,6 @@
 ###############################################################################
 import libxsmm_utilities
 import sys
-import os
 
 
 if __name__ == "__main__":
@@ -39,35 +38,46 @@ if __name__ == "__main__":
     if (3 < argc):
         precision = int(sys.argv[1])
         threshold = int(sys.argv[2])
-        mnklist = libxsmm_utilities.load_mnklist(sys.argv[3:], threshold)
+        mnklist = libxsmm_utilities.load_mnklist(sys.argv[3:], 0)
 
         print("libxsmm_gemm_descriptor desc;")
         print("libxsmm_xmmfunction func;")
         print("unsigned int hash, indx;")
+        print("#if defined(_MSC_VER)")
+        print("# pragma warning(push)")
+        print("# pragma warning(disable: 4127)")
+        print("#endif")
         for mnk in mnklist:
             mstr, nstr, kstr, mnkstr = str(mnk[0]), str(mnk[1]), str(mnk[2]), "_".join(map(str, mnk))
             mnksig = "LIBXSMM_LD(" + mstr + ", " + nstr + "), LIBXSMM_LD(" + nstr + ", " + mstr + "), " + kstr
             ldxsig = "LIBXSMM_LD(" + mstr + ", " + nstr + "), " + kstr + ", LIBXSMM_LD(" + mstr + ", " + nstr + ")"
             # prefer registering double-precision kernels when approaching an exhausted registry
             if (1 != precision): # only double-precision
-                print("LIBXSMM_GEMM_DESCRIPTOR(desc, LIBXSMM_ALIGNMENT, LIBXSMM_FLAGS,")
-                print("  " + mnksig + ", " + ldxsig + ",")
-                print("  LIBXSMM_ALPHA, LIBXSMM_BETA, INTERNAL_PREFETCH);")
-                print("LIBXSMM_HASH_FUNCTION_CALL(hash, indx, desc);")
-                print("func.dmm = (libxsmm_dmmfunction)libxsmm_dmm_" + mnkstr + ";")
-                print("internal_register_static_code(&desc, indx, hash, func, result);")
+                print("if (LIBXSMM_GEMM_NO_BYPASS_DIMS(" + mnksig + ") &&")
+                print("    LIBXSMM_GEMM_NO_BYPASS_DIMS(" + ldxsig + ")) {")
+                print("  LIBXSMM_GEMM_DESCRIPTOR(desc, LIBXSMM_ALIGNMENT, LIBXSMM_FLAGS,")
+                print("    " + mnksig + ", " + ldxsig + ",")
+                print("    LIBXSMM_ALPHA, LIBXSMM_BETA, INTERNAL_PREFETCH);")
+                print("    LIBXSMM_HASH_FUNCTION_CALL(hash, indx, desc); func.dmm = (libxsmm_dmmfunction)libxsmm_dmm_" + mnkstr + ";")
+                print("  internal_register_static_code(&desc, indx, hash, func, result);")
+                print("}")
         for mnk in mnklist:
             mstr, nstr, kstr, mnkstr = str(mnk[0]), str(mnk[1]), str(mnk[2]), "_".join(map(str, mnk))
             mnksig = "LIBXSMM_LD(" + mstr + ", " + nstr + "), LIBXSMM_LD(" + nstr + ", " + mstr + "), " + kstr
             ldxsig = "LIBXSMM_LD(" + mstr + ", " + nstr + "), " + kstr + ", LIBXSMM_LD(" + mstr + ", " + nstr + ")"
             # prefer registering double-precision kernels when approaching an exhausted registry
             if (2 != precision): # only single-precision
-                print("LIBXSMM_GEMM_DESCRIPTOR(desc, LIBXSMM_ALIGNMENT, LIBXSMM_FLAGS | LIBXSMM_GEMM_FLAG_F32PREC,")
-                print("  " + mnksig + ", " + ldxsig + ",")
-                print("  LIBXSMM_ALPHA, LIBXSMM_BETA, INTERNAL_PREFETCH);")
-                print("LIBXSMM_HASH_FUNCTION_CALL(hash, indx, desc);")
-                print("func.smm = (libxsmm_smmfunction)libxsmm_smm_" + mnkstr + ";")
-                print("internal_register_static_code(&desc, indx, hash, func, result);")
+                print("if (LIBXSMM_GEMM_NO_BYPASS_DIMS(" + mnksig + ") &&")
+                print("    LIBXSMM_GEMM_NO_BYPASS_DIMS(" + ldxsig + ")) {")
+                print("  LIBXSMM_GEMM_DESCRIPTOR(desc, LIBXSMM_ALIGNMENT, LIBXSMM_FLAGS | LIBXSMM_GEMM_FLAG_F32PREC,")
+                print("    " + mnksig + ", " + ldxsig + ",")
+                print("    LIBXSMM_ALPHA, LIBXSMM_BETA, INTERNAL_PREFETCH);")
+                print("  LIBXSMM_HASH_FUNCTION_CALL(hash, indx, desc); func.smm = (libxsmm_smmfunction)libxsmm_smm_" + mnkstr + ";")
+                print("  internal_register_static_code(&desc, indx, hash, func, result);")
+                print("}")
+        print("#if defined(_MSC_VER)")
+        print("# pragma warning(pop)")
+        print("#endif")
     elif (1 < argc):
         print("/* no static code */")
     else:
