@@ -100,7 +100,9 @@ void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
 
   /* append new string */
   /* @TODO using memcpy instead? */
-  strcat(l_new_string, i_code_to_append);
+  if (i_code_to_append != NULL) {
+    strcat(l_new_string, i_code_to_append);
+  }
 
   /* free old memory and overwrite pointer */
   if (l_length_1 > 0)
@@ -964,6 +966,9 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
     case LIBXSMM_ERR_INVALID_GEMM_CONFIG:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid GEMM config in setup detected!\n" );
       break;
+    case LIBXSMM_ERR_UNIQUE_VAL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: for sparse-A in reg: too many values in A!\n" );
+      break;
     /* default, we didn't don't know what happend */
     default:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: an unknown error occured!\n" );
