@@ -28,11 +28,6 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include <libxsmm_generator.h>
 #include <libxsmm_macros.h>
 #include "generator_common.h"
@@ -41,7 +36,14 @@
 #include "generator_gemm_imci_avx512.h"
 #include "generator_gemm_noarch.h"
 
+#include <stdlib.h>
+#include <string.h>
+#include <assert.h>
+#include <stdio.h>
+
+
 /* @TODO change int based architecture value */
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_kernel( libxsmm_generated_code*         io_generated_code,
                                     const libxsmm_gemm_descriptor* i_xgemm_desc,
                                     const char*                     i_arch ) {
@@ -143,6 +145,8 @@ void libxsmm_generator_gemm_kernel( libxsmm_generated_code*         io_generated
   libxsmm_generator_gemm_add_flop_counter( io_generated_code, i_xgemm_desc );
 }
 
+
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_inlineasm(const char*                     i_file_out,
                                        const char*                     i_routine_name,
                                        const libxsmm_gemm_descriptor* i_xgemm_desc,
@@ -183,6 +187,7 @@ void libxsmm_generator_gemm_inlineasm(const char*                     i_file_out
   {
     FILE *const l_file_handle = fopen( i_file_out, "a" );
     if ( l_file_handle != NULL ) {
+      assert(l_generated_code.generated_code != NULL);
       fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
@@ -195,6 +200,8 @@ void libxsmm_generator_gemm_inlineasm(const char*                     i_file_out
   free( l_generated_code.generated_code );
 }
 
+
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_directasm(const char*                     i_file_out,
                                        const char*                     i_routine_name,
                                        const libxsmm_gemm_descriptor* i_xgemm_desc,
@@ -230,6 +237,7 @@ void libxsmm_generator_gemm_directasm(const char*                     i_file_out
   {
     FILE *const l_file_handle = fopen( i_file_out, "w" );
     if ( l_file_handle != NULL ) {
+      assert(l_generated_code.generated_code != NULL);
       fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
