@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -31,20 +31,20 @@
 
 #include "generator_gemm_imci_microkernel.h"
 #include "generator_x86_instructions.h"
-#include <libxsmm_macros.h>
+#include <libxsmm_intrinsics_x86.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*             io_generated_code,
-                                               const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                               const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                               const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                               const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                                const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                               const unsigned int                  i_n_blocking,
-                                               const unsigned int                  i_k_blocking,
-                                               const int                           i_offset )
+                                               const unsigned int                 i_n_blocking,
+                                               const unsigned int                 i_k_blocking,
+                                               const int                          i_offset )
 {
   unsigned int l_n;
   unsigned int l_k;
@@ -55,7 +55,8 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
     return;
   }
   if ( (i_offset >= 0) && (i_k_blocking != 1) ) {
-    fprintf(stderr, "LIBXSMM WARNING, libxsmm_generator_gemm_avx512_microkernel: i_k_blocking is ignored as offset is >=0\n");
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_K_BLOCK );
+    return;
   }
 #endif
 
@@ -146,7 +147,7 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
                                  i_k_blocking * i_micro_kernel_config->datatype_size * i_xgemm_desc->lda );
 
     /* advance pointers of B only when we are not fully unrolling K*/
-    if ( i_k_blocking < i_xgemm_desc->k ) {
+    if ( i_k_blocking < (unsigned int)i_xgemm_desc->k ) {
       /* advance pointers of B */
       libxsmm_x86_instruction_alu_imm( io_generated_code,
                                    i_micro_kernel_config->alu_add_instruction,
@@ -156,11 +157,12 @@ void libxsmm_generator_gemm_imci_microkernel( libxsmm_generated_code*
   }
 }
 
-unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*            io_generated_code,
+LIBXSMM_INTERNAL_API_DEFINITION
+unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*             io_generated_code,
                                                         libxsmm_loop_label_tracker*        io_loop_label_tracker,
                                                         const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
                                                         const libxsmm_micro_kernel_config* i_micro_kernel_config,
-                                                        const libxsmm_gemm_descriptor*    i_xgemm_desc,
+                                                        const libxsmm_gemm_descriptor*     i_xgemm_desc,
                                                         const char*                        i_arch,
                                                         unsigned int                       i_n_blocking ) {
   const unsigned int l_k_blocking = 8;
@@ -178,8 +180,8 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
                                                 i_n_blocking,
                                                 i_xgemm_desc->k,
                                                 -1 );
-  } else if ( (i_xgemm_desc->k % l_k_blocking == 0) && (i_xgemm_desc->k >= l_k_threshold) ) {
-    if (i_xgemm_desc->k != l_k_blocking) {
+  } else if ( (i_xgemm_desc->k % l_k_blocking == 0) && (l_k_threshold <= (unsigned int)i_xgemm_desc->k) ) {
+    if (l_k_blocking != (unsigned int)i_xgemm_desc->k) {
       libxsmm_generator_gemm_header_kloop( io_generated_code, io_loop_label_tracker, i_gp_reg_mapping, i_micro_kernel_config,
                                             i_micro_kernel_config->vector_length, l_k_blocking);
     }
@@ -192,7 +194,7 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
                                                 l_k_blocking,
                                                 -1 );
 
-    if (i_xgemm_desc->k != l_k_blocking) {
+    if (l_k_blocking != (unsigned int)i_xgemm_desc->k) {
       libxsmm_generator_gemm_footer_kloop( io_generated_code, io_loop_label_tracker, i_gp_reg_mapping, i_micro_kernel_config,
                                            i_xgemm_desc, i_micro_kernel_config->vector_length, i_xgemm_desc->k, 1 );
     }
@@ -214,7 +216,7 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
       libxsmm_generator_gemm_footer_kloop( io_generated_code, io_loop_label_tracker, i_gp_reg_mapping, i_micro_kernel_config,
                                              i_xgemm_desc, i_micro_kernel_config->vector_length, l_max_blocked_k, 0 );
     }
-    for ( l_k = l_max_blocked_k; l_k < i_xgemm_desc->k; l_k++) {
+    for ( l_k = l_max_blocked_k; l_k < (unsigned int)i_xgemm_desc->k; l_k++) {
       libxsmm_generator_gemm_imci_microkernel( io_generated_code,
                                                 i_gp_reg_mapping,
                                                 i_micro_kernel_config,
@@ -241,6 +243,7 @@ unsigned int libxsmm_generator_gemm_imci_kernel_kloop( libxsmm_generated_code*
   return l_k_unrolled;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated_code,
                                         const unsigned int      i_instruction_set,
                                         const unsigned int      i_vmove_instr,
@@ -290,12 +293,13 @@ void libxsmm_x86_instruction_vec_move_imci( libxsmm_generated_code* io_generated
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_generated_code,
-                                          const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                          const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                          const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                          const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                           const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                          const unsigned int                  i_m_blocking,
-                                          const unsigned int                  i_n_blocking ) {
+                                          const unsigned int                 i_m_blocking,
+                                          const unsigned int                 i_n_blocking ) {
   /* deriving register blocking from kernel config */
   unsigned int l_m_blocking = i_m_blocking/i_micro_kernel_config->vector_length;
   /* start register of accumulator */
@@ -363,12 +367,13 @@ void libxsmm_generator_gemm_load_C_imci( libxsmm_generated_code*             io_
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_store_C_imci( libxsmm_generated_code*             io_generated_code,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                            const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                           const unsigned int                  i_m_blocking,
-                                           const unsigned int                  i_n_blocking )
+                                           const unsigned int                 i_m_blocking,
+                                           const unsigned int                 i_n_blocking )
 {
   /* deriving register blocking from kernel config */
   unsigned int l_m_blocking = i_m_blocking/i_micro_kernel_config->vector_length;
