@@ -28,14 +28,15 @@
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "generator_common.h"
 #include "generator_spgemm_csr_reader.h"
 
+#include <stdlib.h>
+#include <string.h>
+#include <assert.h>
+#include <stdio.h>
+
+
 LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
                                 const char*             i_csr_file_in,
@@ -115,11 +116,11 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
         /* adjust numbers to zero termination */
         l_row--;
         l_column--;
-        /* add these values to row and value strucuture */
+        /* add these values to row and value structure */
         (*o_column_idx)[l_i] = l_column;
         (*o_values)[l_i] = l_value;
         l_i++;
-        /* handle columns, set id to onw for this column, yeah we need to hanle empty columns */
+        /* handle columns, set id to own for this column, yeah we need to handle empty columns */
         l_row_idx_id[l_row] = 1;
         (*o_row_idx)[l_row+1] = l_i;
       }
@@ -129,13 +130,14 @@ void libxsmm_sparse_csr_reader( libxsmm_generated_code* io_generated_code,
   /* close mtx file */
   fclose( l_csr_file_handle );
 
-  /* check if we read a file which was consitent */
+  /* check if we read a file which was consistent */
   if ( l_i != (*o_element_count) ) {
     libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_CSR_LEN );
     return;
   }
 
   /* let's handle empty rows */
+  assert(NULL != l_row_idx_id);
   for ( l_i = 0; l_i < (*o_row_count); l_i++) {
     if ( l_row_idx_id[l_i] == 0 ) {
       (*o_row_idx)[l_i+1] = (*o_row_idx)[l_i];
