@@ -55,7 +55,7 @@
 
 #if defined(LIBXSMM_BUILD) && !defined(LIBXSMM_GEMM_DIFF_NOINLINE)
 # define LIBXSMM_GEMM_DIFF_API LIBXSMM_INLINE LIBXSMM_RETARGETABLE
-# define LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_GEMM_DIFF_API
+# define LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_GEMM_DIFF_API LIBXSMM_ATTRIBUTE_UNUSED
 #else
 # define LIBXSMM_GEMM_DIFF_API LIBXSMM_API
 # define LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_API_DEFINITION
