@@ -41,9 +41,10 @@
 #endif
 #include <assert.h>
 #include <stdlib.h>
+#include <string.h>
 #include <stdio.h>
-#if !defined(NDEBUG)
-# include <string.h>
+#if defined(__TBB)
+# include <tbb/scalable_allocator.h>
 #endif
 #if defined(_WIN32)
 # include <windows.h>
@@ -73,7 +74,6 @@
 #   define LIBXSMM_VTUNE_JIT_LOAD iJVM_EVENT_TYPE_METHOD_LOAD_FINISHED
 # endif
 # define LIBXSMM_VTUNE_JIT_UNLOAD iJVM_EVENT_TYPE_METHOD_UNLOAD_START
-# define LIBXSMM_MALLOC_NOCRC
 #endif /*defined(LIBXSMM_VTUNE)*/
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
@@ -86,13 +86,19 @@
 # define LIBXSMM_MALLOC_FALLBACK 0
 #endif
 
-#if defined(NDEBUG) || defined(LIBXSMM_MALLOC_NOCRC)
+#if !defined(LIBXSMM_MALLOC_NOCRC)
+# if defined(NDEBUG)
+#   define LIBXSMM_MALLOC_NOCRC
+# elif !defined(LIBXSMM_BUILD)
+#   define LIBXSMM_MALLOC_NOCRC
+# endif
+#endif
+
+#if !defined(LIBXSMM_MALLOC_NOCRC)
 # include "libxsmm_hash.h"
 # if !defined(LIBXSMM_MALLOC_SEED)
 #   define LIBXSMM_MALLOC_SEED 1051981
 # endif
-#else
-# define LIBXSMM_MALLOC_NOCRC
 #endif
 
 #if !defined(LIBXSMM_MALLOC_ALIGNMAX)
@@ -109,7 +115,8 @@
 
 
 typedef struct LIBXSMM_RETARGETABLE internal_malloc_info_type {
-  void *pointer, *reloc;
+  libxsmm_free_function free;
+  void *context, *pointer, *reloc;
   size_t size;
   int flags;
 #if defined(LIBXSMM_VTUNE)
@@ -156,43 +163,212 @@ LIBXSMM_API_DEFINITION size_t libxsmm_alignment(size_t size, size_t alignment)
 }
 
 
-#if defined(LIBXSMM_VTUNE)
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_get_vtune_jitdesc(const volatile void* code,
-  unsigned int code_id, size_t code_size, const char* code_name,
-  LIBXSMM_VTUNE_JIT_DESC_TYPE* desc)
+LIBXSMM_API_DEFINITION int libxsmm_xset_default_allocator(LIBXSMM_LOCK_TYPE* lock,
+  void* context, libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn)
 {
-  assert(0 != code && 0 != code_id && 0 != code_size && 0 != desc);
-  desc->method_id = code_id;
-  /* incorrect constness (method_name) */
-  desc->method_name = (char*)code_name;
-  /* incorrect constness (method_load_address) */
-  desc->method_load_address = (void*)code;
-  desc->method_size = code_size;
-  desc->line_number_size = 0;
-  desc->line_number_table = NULL;
-  desc->class_file_name = NULL;
-  desc->source_file_name = NULL;
-# if (2 == LIBXSMM_VTUNE_JITVERSION)
-  desc->module_name = "libxsmm.jit";
-# endif
-}
+  int result = EXIT_SUCCESS;
+  if (0 != lock) {
+    LIBXSMM_INIT
+    LIBXSMM_LOCK_ACQUIRE(lock);
+  }
+  if (0 != malloc_fn.function && 0 != free_fn.function) {
+    libxsmm_default_allocator_context = context;
+    libxsmm_default_malloc_fn = malloc_fn;
+    libxsmm_default_free_fn = free_fn;
+  }
+  else {
+    void* internal_allocator = 0;
+    libxsmm_malloc_function internal_malloc_fn;
+    libxsmm_free_function internal_free_fn;
+#if defined(__TBB)
+    internal_allocator = 0;
+    internal_malloc_fn.function = scalable_malloc;
+    internal_free_fn.function = scalable_free;
+#else
+    internal_allocator = 0;
+    internal_malloc_fn.function = malloc;
+    internal_free_fn.function = free;
 #endif
+    if (0 == malloc_fn.function && 0 == free_fn.function) {
+      libxsmm_default_allocator_context = internal_allocator;
+      libxsmm_default_malloc_fn = internal_malloc_fn;
+      libxsmm_default_free_fn = internal_free_fn;
+    }
+    else { /* invalid allocator */
+      static int error_once = 0;
+      if (0 != libxsmm_verbosity /* library code is expected to be mute */
+        && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+      {
+        fprintf(stderr, "LIBXSMM: allocator setup without malloc or free function!\n");
+      }
+      /* keep any valid (previously instantiated) default allocator */
+      if (0 == libxsmm_default_malloc_fn.function || 0 == libxsmm_default_free_fn.function) {
+        libxsmm_default_allocator_context = internal_allocator;
+        libxsmm_default_malloc_fn = internal_malloc_fn;
+        libxsmm_default_free_fn = internal_free_fn;
+      }
+      result = EXIT_FAILURE;
+    }
+  }
+  if (0 != lock) {
+    LIBXSMM_LOCK_RELEASE(lock);
+  }
+  assert(EXIT_SUCCESS == result);
+  return result;
+}
 
 
-LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_malloc_info_type* internal_malloc_info(const volatile void* memory)
+LIBXSMM_API_DEFINITION int libxsmm_xget_default_allocator(LIBXSMM_LOCK_TYPE* lock,
+  void** context, libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn)
 {
-  internal_malloc_info_type* result = (internal_malloc_info_type*)
+  int result = EXIT_SUCCESS;
+  if (0 != context || 0 != malloc_fn || 0 != free_fn) {
+    if (0 != lock) {
+      LIBXSMM_INIT
+      LIBXSMM_LOCK_ACQUIRE(lock);
+    }
+    if (context) *context = libxsmm_default_allocator_context;
+    if (0 != malloc_fn) *malloc_fn = libxsmm_default_malloc_fn;
+    if (0 != free_fn) *free_fn = libxsmm_default_free_fn;
+    if (0 != lock) {
+      LIBXSMM_LOCK_RELEASE(lock);
+    }
+  }
+  else if (0 != libxsmm_verbosity) { /* library code is expected to be mute */
+    static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+      fprintf(stderr, "LIBXSMM: invalid signature used to get the default memory allocator!\n");
+    }
+    result = EXIT_FAILURE;
+  }
+  assert(EXIT_SUCCESS == result);
+  return result;
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_xset_scratch_allocator(LIBXSMM_LOCK_TYPE* lock,
+  void* context, libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn)
+{
+  int result = EXIT_SUCCESS;
+  static int error_once = 0;
+  if (0 != lock) {
+    LIBXSMM_INIT
+    LIBXSMM_LOCK_ACQUIRE(lock);
+  }
+  /* make sure the default allocator is setup before adopting it eventually */
+  if (0 == libxsmm_default_malloc_fn.function || 0 == libxsmm_default_free_fn.function) {
+    const libxsmm_malloc_function null_malloc_fn = { 0 };
+    const libxsmm_free_function null_free_fn = { 0 };
+    libxsmm_xset_default_allocator(lock, 0/*context*/, null_malloc_fn, null_free_fn);
+  }
+  if (0 == malloc_fn.function && 0 == free_fn.function) { /* adopt default allocator */
+    libxsmm_scratch_allocator_context = libxsmm_default_allocator_context;
+    libxsmm_scratch_malloc_fn = libxsmm_default_malloc_fn;
+    libxsmm_scratch_free_fn = libxsmm_default_free_fn;
+  }
+  else if (0 != malloc_fn.function) {
+    if (0 == free_fn.function
+      && /*warning*/(1 < libxsmm_verbosity || 0 > libxsmm_verbosity)
+      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
+      fprintf(stderr, "LIBXSMM: scratch allocator setup without free function!\n");
+    }
+    libxsmm_scratch_allocator_context = context;
+    libxsmm_scratch_malloc_fn = malloc_fn;
+    libxsmm_scratch_free_fn = free_fn; /* NULL allowed */
+  }
+  else { /* invalid scratch allocator */
+    if (0 != libxsmm_verbosity /* library code is expected to be mute */
+      && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
+      fprintf(stderr, "LIBXSMM: invalid scratch allocator (default used)!\n");
+    }
+    /* keep any valid (previously instantiated) scratch allocator */
+    if (0 == libxsmm_scratch_malloc_fn.function) {
+      libxsmm_scratch_allocator_context = libxsmm_default_allocator_context;
+      libxsmm_scratch_malloc_fn = libxsmm_default_malloc_fn;
+      libxsmm_scratch_free_fn = libxsmm_default_free_fn;
+    }
+    result = EXIT_FAILURE;
+  }
+  if (0 != lock) {
+    LIBXSMM_LOCK_RELEASE(lock);
+  }
+  assert(EXIT_SUCCESS == result);
+  return result;
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_xget_scratch_allocator(LIBXSMM_LOCK_TYPE* lock,
+  void** context, libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn)
+{
+  int result = EXIT_SUCCESS;
+  if (0 != context || 0 != malloc_fn || 0 != free_fn) {
+    if (0 != lock) {
+      LIBXSMM_INIT
+      LIBXSMM_LOCK_ACQUIRE(lock);
+    }
+    if (context) *context = libxsmm_scratch_allocator_context;
+    if (0 != malloc_fn) *malloc_fn = libxsmm_scratch_malloc_fn;
+    if (0 != free_fn) *free_fn = libxsmm_scratch_free_fn;
+    if (0 != lock) {
+      LIBXSMM_LOCK_RELEASE(lock);
+    }
+  }
+  else if (0 != libxsmm_verbosity) { /* library code is expected to be mute */
+    static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+      fprintf(stderr, "LIBXSMM: invalid signature used to get the scratch memory allocator!\n");
+    }
+    result = EXIT_FAILURE;
+  }
+  assert(EXIT_SUCCESS == result);
+  return result;
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_set_default_allocator(void* context,
+  libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn)
+{
+  return libxsmm_xset_default_allocator(&libxsmm_lock_global, context, malloc_fn, free_fn);
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_get_default_allocator(void** context,
+  libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn)
+{
+  return libxsmm_xget_default_allocator(&libxsmm_lock_global, context, malloc_fn, free_fn);
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_set_scratch_allocator(void* context,
+  libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn)
+{
+  return libxsmm_xset_scratch_allocator(&libxsmm_lock_global, context, malloc_fn, free_fn);
+}
+
+
+LIBXSMM_API_DEFINITION int libxsmm_get_scratch_allocator(void** context,
+  libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn)
+{
+  return libxsmm_xget_scratch_allocator(&libxsmm_lock_global, context, malloc_fn, free_fn);
+}
+
+
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE internal_malloc_info_type* internal_malloc_info(const void* memory)
+{
+  internal_malloc_info_type *const result = (internal_malloc_info_type*)
     (0 != memory ? (((const char*)memory) - sizeof(internal_malloc_info_type)) : 0);
 #if defined(LIBXSMM_MALLOC_NOCRC)
   return result;
 #else /* calculate checksum over info */
   return (0 != result && result->hash == libxsmm_crc32(result, /* info size minus actual hash value */
-    sizeof(internal_malloc_info_type) - sizeof(unsigned int), LIBXSMM_MALLOC_SEED)) ? result : 0;
+    (unsigned int)(((char*)&result->hash) - ((char*)result)), LIBXSMM_MALLOC_SEED)) ? result : 0;
 #endif
 }
 
 
-LIBXSMM_API_DEFINITION int libxsmm_malloc_info(const volatile void* memory, size_t* size, int* flags, void** extra)
+LIBXSMM_API_DEFINITION int libxsmm_malloc_info(const void* memory, size_t* size, int* flags, void** extra)
 {
   int result = EXIT_SUCCESS;
 #if !defined(NDEBUG) || !defined(LIBXSMM_MALLOC_NOCRC)
@@ -209,7 +385,9 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_info(const volatile void* memory, size
     else {
       if (0 != memory) {
 #if !defined(LIBXSMM_MALLOC_NOCRC)
-        if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+        if (0 != libxsmm_verbosity /* library code is expected to be mute */
+         && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+        {
           fprintf(stderr, "LIBXSMM: checksum error for memory buffer %p!\n", memory);
         }
 #endif
@@ -222,14 +400,14 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_info(const volatile void* memory, size
   }
 #if !defined(NDEBUG)
   else {
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+    if (0 != libxsmm_verbosity /* library code is expected to be mute */
+     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
       fprintf(stderr, "LIBXSMM: attachment error for memory buffer %p!\n", memory);
     }
     result = EXIT_FAILURE;
   }
-# if defined(LIBXSMM_MALLOC_NOCRC)
   assert(EXIT_SUCCESS == result);
-# endif
 #endif
   return result;
 }
@@ -283,24 +461,34 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void* internal_xmap(const char* dir, size_t
 #endif /*!defined(_WIN32)*/
 
 
-LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int alignment,
+LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, size_t alignment,
   int flags, const void* extra, size_t extra_size)
 {
   int result = EXIT_SUCCESS;
   if (memory) {
     if (0 < size) {
       const size_t internal_size = size + extra_size + sizeof(internal_malloc_info_type);
+      /* ATOMIC BEGIN: this region should be atomic/locked */
+        void* context = libxsmm_default_allocator_context;
+        libxsmm_malloc_function malloc_fn = libxsmm_default_malloc_fn;
+        libxsmm_free_function free_fn = libxsmm_default_free_fn;
+      /* ATOMIC END: this region should be atomic */
       size_t alloc_alignment = 0, alloc_size = 0;
       void *alloc_failed = 0, *buffer = 0, *reloc = 0;
-#if !defined(NDEBUG)
       static int error_once = 0;
-#endif
+      if (0 != (LIBXSMM_MALLOC_FLAG_SCRATCH & flags)) {
+        context = libxsmm_scratch_allocator_context;
+        malloc_fn = libxsmm_scratch_malloc_fn;
+        free_fn = libxsmm_scratch_free_fn;
+      }
       flags |= LIBXSMM_MALLOC_FLAG_RW; /* normalize given flags since flags=0 is accepted as well */
 #if !defined(LIBXSMM_MALLOC_MMAP)
       if (0 == (LIBXSMM_MALLOC_FLAG_X & flags) && 0 == (LIBXSMM_MALLOC_FLAG_MMAP & flags)) {
-        alloc_alignment = 0 <= alignment ? libxsmm_alignment(size, (size_t)alignment) : ((size_t)(-alignment));
+        alloc_alignment = (0 == alignment ? libxsmm_alignment(size, alignment) : alignment);
         alloc_size = internal_size + alloc_alignment - 1;
-        buffer = malloc(alloc_size);
+        buffer = 0 != malloc_fn.function
+          ? (0 == context ? malloc_fn.function(alloc_size) : malloc_fn.ctx_form(context, alloc_size))
+          : 0;
       }
       else
 #endif
@@ -308,7 +496,7 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
 #if defined(_WIN32)
         const int xflags = (0 != (LIBXSMM_MALLOC_FLAG_X & flags) ? PAGE_EXECUTE_READWRITE : PAGE_READWRITE);
         if ((LIBXSMM_MALLOC_ALIGNMAX * LIBXSMM_MALLOC_ALIGNFCT) > size) {
-          alloc_alignment = 0 <= alignment ? libxsmm_alignment(size, (size_t)alignment) : ((size_t)(-alignment));
+          alloc_alignment = (0 == alignment ? libxsmm_alignment(size, alignment) : alignment);
           alloc_size = internal_size + alloc_alignment - 1;
           buffer = VirtualAlloc(0, alloc_size, MEM_RESERVE | MEM_COMMIT, xflags);
         }
@@ -316,7 +504,7 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
           HANDLE process_token;
           const SIZE_T alloc_alignmax = GetLargePageMinimum();
           /* respect user-requested alignment */
-          alloc_alignment = 0 == alignment ? alloc_alignmax : libxsmm_lcm((size_t)LIBXSMM_ABS(alignment), alloc_alignmax);
+          alloc_alignment = 0 == alignment ? alloc_alignmax : libxsmm_lcm(alignment, alloc_alignmax);
           alloc_size = LIBXSMM_UP2(internal_size, alloc_alignment); /* assume that alloc_alignment is POT */
           if (TRUE == OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &process_token)) {
             TOKEN_PRIVILEGES tp;
@@ -333,7 +521,7 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
             CloseHandle(process_token);
           }
           if (alloc_failed == buffer) { /* retry allocation with regular page size */
-            alloc_alignment = 0 <= alignment ? libxsmm_alignment(size, (size_t)alignment) : ((size_t)(-alignment));
+            alloc_alignment = (0 == alignment ? libxsmm_alignment(size, alignment) : alignment);
             alloc_size = internal_size + alloc_alignment - 1;
             buffer = VirtualAlloc(0, alloc_size, MEM_RESERVE | MEM_COMMIT, xflags);
           }
@@ -342,7 +530,9 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
           flags |= LIBXSMM_MALLOC_FLAG_MMAP; /* select the corresponding deallocation */
         }
         else if (0 == (LIBXSMM_MALLOC_FLAG_MMAP & flags)) { /* fall-back allocation */
-          buffer = malloc(alloc_size);
+          buffer = 0 != malloc_fn.function
+            ? (0 == context ? malloc_fn.function(alloc_size) : malloc_fn.ctx_form(context, alloc_size))
+            : 0;
         }
 #else /* !defined(_WIN32) */
         int xflags = 0
@@ -375,37 +565,49 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
           }
         }
 # endif
-        alloc_alignment = 0 <= alignment ? libxsmm_alignment(size, (size_t)alignment) : ((size_t)(-alignment));
+        alloc_alignment = (0 == alignment ? libxsmm_alignment(size, alignment) : alignment);
         alloc_size = internal_size + alloc_alignment - 1;
         alloc_failed = MAP_FAILED;
         if (0 == (LIBXSMM_MALLOC_FLAG_X & flags)) {
           buffer = mmap(0, alloc_size, PROT_READ | PROT_WRITE, MAP_PRIVATE | LIBXSMM_MAP_ANONYMOUS | xflags, -1, 0);
         }
         else {
-          static LIBXSMM_TLS int fallback = LIBXSMM_MALLOC_FALLBACK;
+          static LIBXSMM_TLS int fallback = -1;
+          if (0 > fallback) { /* initialize fallback allocation method */
+            const char *const env = getenv("LIBXSMM_SE");
+            fallback = (0 == env || 0 == *env || 0 != atoi(env)) ? LIBXSMM_MALLOC_FALLBACK : 4;
+          }
           if (0 == fallback) {
             buffer = internal_xmap("/tmp", alloc_size, xflags, &reloc);
             if (alloc_failed == buffer) fallback = 1;
           }
-          if (1 == fallback) { /* 2nd try */
-            buffer = internal_xmap(".", alloc_size, xflags, &reloc);
-            if (alloc_failed == buffer) fallback = 2;
-          }
-          if (2 == fallback) { /* 3rd try */
-            buffer = internal_xmap(getenv("HOME"), alloc_size, xflags, &reloc);
-            if (alloc_failed == buffer) fallback = 3;
-          }
-          if (3 == fallback) { /* 4th try */
-            buffer = internal_xmap(getenv("JITDUMPDIR"), alloc_size, xflags, &reloc);
-            if (alloc_failed == buffer) fallback = 4;
-          }
-          if (4 == fallback) { /* 5th try */
-            buffer = mmap(0, alloc_size, PROT_READ | PROT_WRITE | PROT_EXEC,
-              MAP_PRIVATE | LIBXSMM_MAP_ANONYMOUS | xflags, -1, 0);
-            if (alloc_failed == buffer) fallback = 5;
-          }
-          if (5 == fallback && alloc_failed != buffer) { /* final */
-            buffer = alloc_failed; /* trigger fall-back */
+          if (1 <= fallback) { /* continue with fall-back */
+            if (1 == fallback) { /* 2nd try */
+              buffer = internal_xmap(".", alloc_size, xflags, &reloc);
+              if (alloc_failed == buffer) fallback = 2;
+            }
+            if (2 <= fallback) { /* continue with fall-back */
+              if (2 == fallback) { /* 3rd try */
+                buffer = internal_xmap(getenv("HOME"), alloc_size, xflags, &reloc);
+                if (alloc_failed == buffer) fallback = 3;
+              }
+              if (3 <= fallback) { /* continue with fall-back */
+                if (3 == fallback) { /* 4th try */
+                  buffer = internal_xmap(getenv("JITDUMPDIR"), alloc_size, xflags, &reloc);
+                  if (alloc_failed == buffer) fallback = 4;
+                }
+                if (4 <= fallback) { /* continue with fall-back */
+                  if (4 == fallback) { /* 5th try */
+                    buffer = mmap(0, alloc_size, PROT_READ | PROT_WRITE | PROT_EXEC,
+                      MAP_PRIVATE | LIBXSMM_MAP_ANONYMOUS | xflags, -1, 0);
+                    if (alloc_failed == buffer) fallback = 5;
+                  }
+                  if (5 == fallback && alloc_failed != buffer) { /* final */
+                    buffer = alloc_failed; /* trigger fall-back */
+                  }
+                }
+              }
+            }
           }
         }
         if (alloc_failed != buffer) {
@@ -414,7 +616,9 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
         }
         else {
           if (0 == (LIBXSMM_MALLOC_FLAG_MMAP & flags)) { /* fall-back allocation */
-            buffer = malloc(alloc_size);
+            buffer = 0 != malloc_fn.function
+              ? (0 == context ? malloc_fn.function(alloc_size) : malloc_fn.ctx_form(context, alloc_size))
+              : 0;
             reloc = buffer;
           }
           else {
@@ -430,37 +634,37 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
         char *const aligned = LIBXSMM_ALIGN(((char*)buffer) + extra_size + sizeof(internal_malloc_info_type), alloc_alignment);
         internal_malloc_info_type *const info = (internal_malloc_info_type*)(aligned - sizeof(internal_malloc_info_type));
         assert((aligned + size) <= (((char*)buffer) + alloc_size));
-        if (0 < extra_size && 0 != extra) {
-          const char *const src = (const char*)extra;
-          char *const dst = (char*)buffer;
-          size_t i;
-#if defined(_MSC_VER) && (1900 <= _MSC_VER)
-#         pragma warning(suppress: 6386)
-#endif
-          for (i = 0; i < extra_size; ++i) dst[i] = src[i];
-        }
+        if (0 != extra) memcpy(buffer, extra, extra_size);
 #if !defined(NDEBUG)
         else if (0 == extra && 0 != extra_size) {
           result = EXIT_FAILURE;
         }
 #endif
-        info->reloc = reloc;
+        if (0 == (LIBXSMM_MALLOC_FLAG_MMAP & flags)) {
+          info->context = context;
+          info->free = free_fn;
+        }
+        else {
+          info->free.function = 0;
+          info->context = 0;
+        }
         info->pointer = buffer;
+        info->reloc = reloc;
         info->size = size;
         info->flags = flags;
 #if !defined(LIBXSMM_MALLOC_NOCRC) /* calculate checksum over info */
         info->hash = libxsmm_crc32(info, /* info size minus actual hash value */
-          sizeof(internal_malloc_info_type) - sizeof(unsigned int), LIBXSMM_MALLOC_SEED);
+          (unsigned int)(((char*)&info->hash) - ((char*)info)), LIBXSMM_MALLOC_SEED);
 #endif
         *memory = aligned;
       }
       else {
-#if !defined(NDEBUG) /* library code is expected to be mute */
-        if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+        if (0 != libxsmm_verbosity /* library code is expected to be mute */
+         && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+        {
           fprintf(stderr, "LIBXSMM: memory allocation error for size %llu with flags=%i!\n",
             (unsigned long long)alloc_size, flags);
         }
-#endif
         result = EXIT_FAILURE;
       }
     }
@@ -468,72 +672,89 @@ LIBXSMM_API_DEFINITION int libxsmm_xmalloc(void** memory, size_t size, int align
       *memory = 0;
     }
   }
-#if !defined(NDEBUG)
   else if (0 != size) {
     result = EXIT_FAILURE;
   }
-#endif
   assert(EXIT_SUCCESS == result);
   return result;
 }
 
 
-LIBXSMM_API_DEFINITION int libxsmm_xfree(const volatile void* memory)
+LIBXSMM_API_DEFINITION int libxsmm_xfree(const void* memory)
 {
   /*const*/ internal_malloc_info_type *const info = internal_malloc_info(memory);
   int result = EXIT_SUCCESS;
-# if !defined(NDEBUG) || !defined(LIBXSMM_MALLOC_NOCRC)
   static int error_once = 0;
-#endif
   if (0 != info) {
     void *const buffer = info->pointer;
-    assert((0 != buffer || 0 == info->size));
-    if (0 == (LIBXSMM_MALLOC_FLAG_MMAP & info->flags)) {
-      free(buffer);
-    }
-    else {
-#if defined(LIBXSMM_VTUNE)
-      if (0 != (LIBXSMM_MALLOC_FLAG_X & info->flags) && 0 != info->code_id && iJIT_SAMPLING_ON == iJIT_IsProfilingActive()) {
-        iJIT_NotifyEvent(LIBXSMM_VTUNE_JIT_UNLOAD, &info->code_id);
+#if !defined(LIBXSMM_BUILD) /* sanity check */
+    if (0 != buffer || 0 == info->size)
+#endif
+    {
+      assert(0 != buffer || 0 == info->size);
+      if (0 == (LIBXSMM_MALLOC_FLAG_MMAP & info->flags)) {
+        if (0 != info->free.function) {
+          if (0 == info->context) {
+            info->free.function(buffer);
+          }
+          else {
+            info->free.ctx_form(info->context, buffer);
+          }
+        }
       }
+      else {
+#if defined(LIBXSMM_VTUNE)
+        if (0 != (LIBXSMM_MALLOC_FLAG_X & info->flags) && 0 != info->code_id && iJIT_SAMPLING_ON == iJIT_IsProfilingActive()) {
+          iJIT_NotifyEvent(LIBXSMM_VTUNE_JIT_UNLOAD, &info->code_id);
+        }
 #endif
 #if defined(_WIN32)
-      result = FALSE != VirtualFree(buffer, 0, MEM_RELEASE) ? EXIT_SUCCESS : EXIT_FAILURE;
+        result = (0 == buffer || FALSE != VirtualFree(buffer, 0, MEM_RELEASE)) ? EXIT_SUCCESS : EXIT_FAILURE;
 #else /* defined(_WIN32) */
-      {
-        const size_t alloc_size = info->size + (((const char*)memory) - ((const char*)buffer));
-        void *const reloc = info->reloc;
-        const int flags = info->flags;
-        if (0 != munmap(buffer, alloc_size)) {
-# if !defined(NDEBUG) /* library code is expected to be mute */
-          if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-            const char *const error_message = strerror(errno);
-            fprintf(stderr, "LIBXSMM: %s (munmap error #%i for range %p+%llu)!\n",
-              error_message, errno, buffer, (unsigned long long)alloc_size);
-          }
-# endif
-          result = EXIT_FAILURE;
-        }
-        if (0 != (LIBXSMM_MALLOC_FLAG_X & flags) && EXIT_SUCCESS == result
-         && 0 != reloc && MAP_FAILED != reloc && buffer != reloc
-         && 0 != munmap(reloc, alloc_size))
         {
-# if !defined(NDEBUG) /* library code is expected to be mute */
-          if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-            const char *const error_message = strerror(errno);
-            fprintf(stderr, "LIBXSMM: %s (munmap error #%i for range %p+%llu)!\n",
-              error_message, errno, reloc, (unsigned long long)alloc_size);
+          const size_t alloc_size = info->size + (((const char*)memory) - ((const char*)buffer));
+          void *const reloc = info->reloc;
+          const int flags = info->flags;
+          if (0 != munmap(buffer, alloc_size)) {
+            if (0 != libxsmm_verbosity /* library code is expected to be mute */
+             && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+            {
+              const char *const error_message = strerror(errno);
+              fprintf(stderr, "LIBXSMM: %s (munmap error #%i for range %p+%llu)!\n",
+                error_message, errno, buffer, (unsigned long long)alloc_size);
+            }
+            result = EXIT_FAILURE;
+          }
+          if (0 != (LIBXSMM_MALLOC_FLAG_X & flags) && EXIT_SUCCESS == result
+           && 0 != reloc && MAP_FAILED != reloc && buffer != reloc
+           && 0 != munmap(reloc, alloc_size))
+          {
+            if (0 != libxsmm_verbosity /* library code is expected to be mute */
+             && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+            {
+              const char *const error_message = strerror(errno);
+              fprintf(stderr, "LIBXSMM: %s (munmap error #%i for range %p+%llu)!\n",
+                error_message, errno, reloc, (unsigned long long)alloc_size);
+            }
+            result = EXIT_FAILURE;
           }
-# endif
-          result = EXIT_FAILURE;
         }
-      }
 #endif
+      }
     }
+#if !defined(LIBXSMM_BUILD)
+    else if ((0 > libxsmm_verbosity || 1 < libxsmm_verbosity) /* library code is expected to be mute */
+     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
+      fprintf(stderr, "LIBXSMM: attempt to release memory from non-matching implementation!\n");
+    }
+#endif
   }
   else if (0 != memory) {
 #if !defined(LIBXSMM_MALLOC_NOCRC)
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+    if (0 != libxsmm_verbosity /* library code is expected to be mute */
+     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
       fprintf(stderr, "LIBXSMM: checksum error for memory buffer %p!\n", memory);
     }
 #endif
@@ -544,13 +765,34 @@ LIBXSMM_API_DEFINITION int libxsmm_xfree(const volatile void* memory)
 }
 
 
+#if defined(LIBXSMM_VTUNE)
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_get_vtune_jitdesc(const void* code,
+  unsigned int code_id, size_t code_size, const char* code_name,
+  LIBXSMM_VTUNE_JIT_DESC_TYPE* desc)
+{
+  assert(0 != code && 0 != code_id && 0 != code_size && 0 != desc);
+  desc->method_id = code_id;
+  /* incorrect constness (method_name) */
+  desc->method_name = (char*)code_name;
+  /* incorrect constness (method_load_address) */
+  desc->method_load_address = (void*)code;
+  desc->method_size = code_size;
+  desc->line_number_size = 0;
+  desc->line_number_table = NULL;
+  desc->class_file_name = NULL;
+  desc->source_file_name = NULL;
+# if (2 == LIBXSMM_VTUNE_JITVERSION)
+  desc->module_name = "libxsmm.jit";
+# endif
+}
+#endif
+
+
 LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const char* name)
 {
   internal_malloc_info_type *const info = 0 != memory ? internal_malloc_info(*memory) : 0;
   int result = EXIT_SUCCESS;
-#if !defined(NDEBUG) || !defined(LIBXSMM_MALLOC_NOCRC)
   static int error_once = 0;
-#endif
   if (0 != info) {
     void *const buffer = info->pointer;
     const size_t size = info->size;
@@ -561,7 +803,7 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const
 #endif
     /* quietly keep the read permission, but eventually revoke write permissions */
     if (0 == (LIBXSMM_MALLOC_FLAG_W & flags) || 0 != (LIBXSMM_MALLOC_FLAG_X & flags)) {
-      const int alignment = (int)(((const char*)(*memory)) - ((const char*)buffer));
+      const size_t alignment = (size_t)(((const char*)(*memory)) - ((const char*)buffer));
       const size_t alloc_size = size + alignment;
       if (0 == (LIBXSMM_MALLOC_FLAG_X & flags)) {
 #if defined(_WIN32)
@@ -618,7 +860,7 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const
           info->reloc = 0;
 # if !defined(LIBXSMM_MALLOC_NOCRC) /* update checksum */
           info->hash = libxsmm_crc32(info, /* info size minus actual hash value */
-            sizeof(internal_malloc_info_type) - sizeof(unsigned int), LIBXSMM_MALLOC_SEED);
+            (unsigned int)(((char*)&info->hash) - ((char*)info)), LIBXSMM_MALLOC_SEED);
 # endif
           /* treat memory protection errors as soft error; ignore return value */
           munmap(buffer, alloc_size);
@@ -628,7 +870,7 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const
         else { /* malloc-based fall-back */
 # if !defined(LIBXSMM_MALLOC_NOCRC) && defined(LIBXSMM_VTUNE) /* update checksum */
           info->hash = libxsmm_crc32(info, /* info size minus actual hash value */
-            sizeof(internal_malloc_info_type) - sizeof(unsigned int), LIBXSMM_MALLOC_SEED);
+            (unsigned int)(((char*)&info->hash) - ((char*)info)), LIBXSMM_MALLOC_SEED);
 # endif
           /* treat memory protection errors as soft error; ignore return value */
           mprotect(buffer, alloc_size/*entire memory region*/, PROT_READ | PROT_EXEC);
@@ -638,18 +880,21 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const
     }
   }
   else if (0 == memory || 0 == *memory) {
-#if !defined(NDEBUG) /* library code is expected to be mute */
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+    if (0 != libxsmm_verbosity /* library code is expected to be mute */
+     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
       fprintf(stderr, "LIBXSMM: libxsmm_malloc_attrib failed because NULL cannot be attributed!\n");
     }
-#endif
     result = EXIT_FAILURE;
   }
   else {
     assert(0 != memory && 0 != *memory);
 #if !defined(LIBXSMM_MALLOC_NOCRC)
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: checksum error for memory buffer %p!\n", *memory);
+    if (0 != libxsmm_verbosity /* library code is expected to be mute */
+     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
+    {
+      fprintf(stderr, "LIBXSMM: checksum error for %s buffer %p!\n",
+        0 != (LIBXSMM_MALLOC_FLAG_X & flags) ? "executable" : "memory", *memory);
     }
 #endif
     result = EXIT_FAILURE;
@@ -659,7 +904,7 @@ LIBXSMM_API_DEFINITION int libxsmm_malloc_attrib(void** memory, int flags, const
 }
 
 
-LIBXSMM_API_DEFINITION void* libxsmm_aligned_malloc(size_t size, int alignment)
+LIBXSMM_API_DEFINITION void* libxsmm_aligned_malloc(size_t size, size_t alignment)
 {
   void* result = 0;
   LIBXSMM_INIT
@@ -668,19 +913,35 @@ LIBXSMM_API_DEFINITION void* libxsmm_aligned_malloc(size_t size, int alignment)
 }
 
 
+LIBXSMM_API_DEFINITION void* libxsmm_aligned_scratch(size_t size, size_t alignment)
+{
+  void* result = 0;
+  LIBXSMM_INIT
+  return 0 == libxsmm_xmalloc(&result, size, alignment, LIBXSMM_MALLOC_FLAG_SCRATCH,
+    0/*extra*/, 0/*extra_size*/) ? result : 0;
+}
+
+
 LIBXSMM_API_DEFINITION void* libxsmm_malloc(size_t size)
 {
   return libxsmm_aligned_malloc(size, 0/*auto*/);
 }
 
 
-LIBXSMM_API_DEFINITION void libxsmm_free(const volatile void* memory)
+LIBXSMM_API_DEFINITION void libxsmm_free(const void* memory)
 {
   libxsmm_xfree(memory);
 }
 
 
-LIBXSMM_API_DEFINITION size_t libxsmm_malloc_size(const volatile void* memory)
+LIBXSMM_API_DEFINITION void libxsmm_release_scratch(size_t* npending)
+{
+  /* TODO: to be implemented */
+  LIBXSMM_UNUSED(npending);
+}
+
+
+LIBXSMM_API_DEFINITION size_t libxsmm_malloc_size(const void* memory)
 {
   size_t size = 0;
   libxsmm_malloc_info(memory, &size, 0/*flags*/, 0/*extra*/);
