@@ -45,29 +45,21 @@
 # pragma offload_attribute(pop)
 #endif
 
-#if !defined(LIBXSMM_SPMDM_MALLOC_INTRINSIC) && !defined(LIBXSMM_INTRINSICS_NONE)
-# define LIBXSMM_SPMDM_MALLOC_INTRINSIC
-#endif
-#if defined(LIBXSMM_SPMDM_MALLOC_INTRINSIC)
-# define LIBXSMM_SPMDM_MALLOC(SIZE, ALIGNMENT) _mm_malloc(SIZE, ALIGNMENT)
-# define LIBXSMM_SPMDM_FREE(BUFFER) _mm_free((void*)(BUFFER))
-#else
-# define LIBXSMM_SPMDM_MALLOC(SIZE, ALIGNMENT) libxsmm_aligned_malloc(SIZE, -(ALIGNMENT))
-# define LIBXSMM_SPMDM_FREE(BUFFER) libxsmm_free(BUFFER)
-#endif
-
 /* Enable/disable specific code paths */
-#if !defined(LIBXSMM_SPMDM_AVX512_CORE)
-# define LIBXSMM_SPMDM_AVX512_CORE
+#if !defined(LIBXSMM_SPMDM_AVX) && !defined(LIBXSMM_INTRINSICS_NONE) && ( \
+     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
+  || (defined(__clang__) && LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH))
+# define LIBXSMM_SPMDM_AVX
 #endif
-#if !defined(LIBXSMM_SPMDM_AVX2)
+#if !defined(LIBXSMM_SPMDM_AVX2) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX) && ( \
+     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
+  || (defined(__clang__) && LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH))
 # define LIBXSMM_SPMDM_AVX2
 #endif
-
-
-#if !defined(LIBXSMM_INTRINSICS_NONE) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE __m256i internal_spmdm_shufmasks_32[256];
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE __m256i internal_spmdm_shufmasks_16[256];
+#if !defined(LIBXSMM_SPMDM_AVX512_CORE) && !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX2) && ( \
+     (!defined(LIBXSMM_INTRINSICS_LEGACY) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)) \
+  || (defined(__clang__) && LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH))
+# define LIBXSMM_SPMDM_AVX512_CORE
 #endif
 
 
@@ -81,15 +73,20 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*internal_spmdm_compute_fp32_thread)
 LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void (*internal_spmdm_compute_bfloat16_thread)(const libxsmm_spmdm_handle*, char, char,
   const uint16_t*, libxsmm_CSR_sparseslice*, const uint16_t*, char, const uint16_t*, float*, int, int, int);
 
+#if defined(LIBXSMM_SPMDM_AVX)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE __m256i* internal_spmdm_shufmasks_32;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE __m256i* internal_spmdm_shufmasks_16;
+#endif
+
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX)
-void internal_spmdm_init_shufmask_avx()
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_init_shufmask_avx()
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
-  && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
-  unsigned int i, j, c, last_bit;
+#if defined(LIBXSMM_SPMDM_AVX)
+  static __m256i spmdm_shufmasks_32[256], spmdm_shufmasks_16[256];
   LIBXSMM_ALIGNED(int temp_shufmasks[8], 64);
   LIBXSMM_ALIGNED(uint16_t temp_shufmasks2[16], 64);
+  unsigned int i, j, c, last_bit;
   int cnt;
   for (i = 0; i < 256; i++) {
     cnt = 0;
@@ -103,9 +100,11 @@ void internal_spmdm_init_shufmask_avx()
       j &= (~(1<<last_bit));
       cnt++;
     }
-    internal_spmdm_shufmasks_32[i] = _mm256_loadu_si256((const __m256i*)temp_shufmasks);
-    internal_spmdm_shufmasks_16[i] = _mm256_loadu_si256((const __m256i*)temp_shufmasks2);
+    spmdm_shufmasks_32[i] = _mm256_loadu_si256((const __m256i*)temp_shufmasks);
+    spmdm_shufmasks_16[i] = _mm256_loadu_si256((const __m256i*)temp_shufmasks2);
   }
+  internal_spmdm_shufmasks_32 = spmdm_shufmasks_32;
+  internal_spmdm_shufmasks_16 = spmdm_shufmasks_16;
 #endif
 }
 
@@ -119,7 +118,7 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_spmdm_allocate_csr_a(libxsmm_s
   size_t sz_block = ((handle->bm + 1)*sizeof(uint16_t) + (handle->bm)*(handle->bk)*sizeof(uint16_t) + (handle->bm)*(handle->bk)*sizeof(float) + sizeof(libxsmm_CSR_sparseslice));
   size_t sz_all_blocks = sz_block * handle->mb * handle->kb;
 
-  char * memory_block = (char *)LIBXSMM_SPMDM_MALLOC( sz_all_blocks, 2097152);
+  char * memory_block = (char *)libxsmm_aligned_scratch(sz_all_blocks, 2097152);
   char * memory_head  = memory_block;
 
   libxsmm_CSR_sparseslice* libxsmm_output_csr_a = (libxsmm_CSR_sparseslice*)(memory_head);
@@ -144,20 +143,20 @@ LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_spmdm_allocate_csr_a(libxsmm_s
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_spmdm_allocate_scratch(libxsmm_spmdm_handle* handle, int max_threads)
 {
-  size_t sz_memory_for_scratch_per_thread = ((handle->bm)*(handle->bn)*sizeof(float) + (handle->bk)*(handle->bn)*sizeof(float))*max_threads, sz_total_memory;
+  size_t sz_memory_for_scratch_per_thread = ((handle->bm)*(handle->bn)*sizeof(float) + (handle->bk)*(handle->bn)*sizeof(float)), sz_total_memory;
   sz_memory_for_scratch_per_thread = (sz_memory_for_scratch_per_thread + 4095)/4096 * 4096;
   sz_total_memory = sz_memory_for_scratch_per_thread * max_threads;
 
-  handle->base_ptr_scratch_B_scratch_C = (char *)LIBXSMM_SPMDM_MALLOC(sz_total_memory, 2097152);
+  handle->base_ptr_scratch_B_scratch_C = (char *)libxsmm_aligned_scratch(sz_total_memory, 2097152);
   handle->memory_for_scratch_per_thread = (int)sz_memory_for_scratch_per_thread;
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_spmdm_deallocate_csr_a(libxsmm_spmdm_handle* handle)
 {
-  LIBXSMM_SPMDM_FREE(handle->base_ptr_scratch_A);
+  libxsmm_free(handle->base_ptr_scratch_A);
   handle->base_ptr_scratch_A= NULL;
-  LIBXSMM_SPMDM_FREE(handle->base_ptr_scratch_B_scratch_C);
+  libxsmm_free(handle->base_ptr_scratch_B_scratch_C);
   handle->base_ptr_scratch_B_scratch_C = NULL;
 }
 
@@ -196,7 +195,7 @@ void internal_spmdm_createSparseSlice_fp32_thread_sw(
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
-void internal_spmdm_createSparseSlice_fp32_thread_avx2(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx2(
   const libxsmm_spmdm_handle* handle,
   char transA,
   const float* A,
@@ -204,25 +203,18 @@ void internal_spmdm_createSparseSlice_fp32_thread_avx2(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX2)
 # include "libxsmm_spmdm_begin_avx2.h"
 # include "template/libxsmm_spmdm_createSparseSlice_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX2) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    }
-  }
-# endif
   internal_spmdm_createSparseSlice_fp32_thread_sw(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
-void internal_spmdm_createSparseSlice_fp32_thread_avx512_core(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_fp32_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
   char transA,
   const float* A,
@@ -230,18 +222,11 @@ void internal_spmdm_createSparseSlice_fp32_thread_avx512_core(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX512_CORE) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX512_CORE)
 # include "libxsmm_spmdm_begin_avx512.h"
 # include "template/libxsmm_spmdm_createSparseSlice_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX512_CORE) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX-512/Core code path!\n");
-    }
-  }
-# endif
   internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
@@ -256,10 +241,16 @@ void libxsmm_spmdm_createSparseSlice_fp32_thread(
   int block_id,
   int tid, int nthreads)
 {
+  /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_createSparseSlice_fp32_thread_avx512_core(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no no need for an indirect call */
+# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_createSparseSlice_fp32_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+# else /* pointer based function call */
+  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
+  internal_spmdm_createSparseSlice_fp32_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+# endif
 #else /* pointer based function call */
   assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
   internal_spmdm_createSparseSlice_fp32_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
@@ -283,7 +274,7 @@ void internal_spmdm_createSparseSlice_bfloat16_thread_sw(
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
-void internal_spmdm_createSparseSlice_bfloat16_thread_avx2(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_avx2(
   const libxsmm_spmdm_handle* handle,
   char transA,
   const uint16_t* A,
@@ -291,25 +282,18 @@ void internal_spmdm_createSparseSlice_bfloat16_thread_avx2(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX2)
 # include "libxsmm_spmdm_begin_avx2.h"
 # include "template/libxsmm_spmdm_createSparseSlice_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX2) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    }
-  }
-# endif
   internal_spmdm_createSparseSlice_bfloat16_thread_sw(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
-void internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
   char transA,
   const uint16_t* A,
@@ -317,18 +301,11 @@ void internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX512_CORE) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX512_CORE)
 # include "libxsmm_spmdm_begin_avx512.h"
 # include "template/libxsmm_spmdm_createSparseSlice_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX512_CORE) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX-512/Core code path!\n");
-    }
-  }
-# endif
   internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
 #endif
 }
@@ -343,10 +320,16 @@ void libxsmm_spmdm_createSparseSlice_bfloat16_thread(
   int block_id,
   int tid, int nthreads)
 {
+  /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
-#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no no need for an indirect call */
+# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_createSparseSlice_bfloat16_thread_avx2(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+# else /* pointer based function call */
+  assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
+  internal_spmdm_createSparseSlice_bfloat16_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
+# endif
 #else /* pointer based function call */
   assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
   internal_spmdm_createSparseSlice_bfloat16_thread(handle, transA, A, libxsmm_output_csr_a, block_id, tid, nthreads);
@@ -375,7 +358,7 @@ void internal_spmdm_compute_fp32_thread_sw(
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
-void internal_spmdm_compute_fp32_thread_avx2(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx2(
   const libxsmm_spmdm_handle* handle,
   char transA,
   char transB,
@@ -388,25 +371,18 @@ void internal_spmdm_compute_fp32_thread_avx2(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX2)
 # include "libxsmm_spmdm_begin_avx2.h"
 # include "template/libxsmm_spmdm_compute_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX2) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    }
-  }
-# endif
   internal_spmdm_compute_fp32_thread_sw(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
 #endif
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
-void internal_spmdm_compute_fp32_thread_avx512_core(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_fp32_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
   char transA,
   char transB,
@@ -419,18 +395,11 @@ void internal_spmdm_compute_fp32_thread_avx512_core(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX512_CORE) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX512_CORE)
 # include "libxsmm_spmdm_begin_avx512.h"
 # include "template/libxsmm_spmdm_compute_fp32_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX512_CORE) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX-512/Core code path!\n");
-    }
-  }
-# endif
   internal_spmdm_compute_fp32_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
 #endif
 }
@@ -450,10 +419,9 @@ void libxsmm_spmdm_compute_fp32_thread(
   int block_id,
   int tid, int nthreads)
 {
+  /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_compute_fp32_thread_avx512_core(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
-#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
-  internal_spmdm_compute_fp32_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
 #else /* pointer based function call */
   assert(0 != internal_spmdm_compute_fp32_thread);
   internal_spmdm_compute_fp32_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
@@ -482,7 +450,7 @@ void internal_spmdm_compute_bfloat16_thread_sw(
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
-void internal_spmdm_compute_bfloat16_thread_avx2(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx2(
   const libxsmm_spmdm_handle* handle,
   char transA,
   char transB,
@@ -495,25 +463,18 @@ void internal_spmdm_compute_bfloat16_thread_avx2(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX2)
 # include "libxsmm_spmdm_begin_avx2.h"
 # include "template/libxsmm_spmdm_compute_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX2) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-    }
-  }
-# endif
   internal_spmdm_compute_bfloat16_thread_sw(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
 #endif
 }
 
 
 LIBXSMM_INLINE LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
-void internal_spmdm_compute_bfloat16_thread_avx512_core(
+LIBXSMM_ATTRIBUTE_UNUSED void internal_spmdm_compute_bfloat16_thread_avx512_core(
   const libxsmm_spmdm_handle* handle,
   char transA,
   char transB,
@@ -526,18 +487,11 @@ void internal_spmdm_compute_bfloat16_thread_avx512_core(
   int block_id,
   int tid, int nthreads)
 {
-#if !defined(LIBXSMM_INTRINSICS_NONE) && defined(LIBXSMM_SPMDM_AVX512_CORE) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if defined(LIBXSMM_SPMDM_AVX512_CORE)
 # include "libxsmm_spmdm_begin_avx512.h"
 # include "template/libxsmm_spmdm_compute_bfloat16_thread.tpl.c"
 # include "libxsmm_spmdm_end.h"
 #else
-# if !defined(NDEBUG) && defined(LIBXSMM_SPMDM_AVX512_CORE) /* library code is expected to be mute */
-  { static int error_once = 0;
-    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
-      fprintf(stderr, "LIBXSMM: unable to enter AVX-512/Core code path!\n");
-    }
-  }
-# endif
   internal_spmdm_compute_bfloat16_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
 #endif
 }
@@ -557,10 +511,16 @@ void libxsmm_spmdm_compute_bfloat16_thread(
   int block_id,
   int tid, int nthreads)
 {
+  /* if highest implemented code path is statically present, no need for an indirect call (function pointer) */
 #if (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_compute_bfloat16_thread_avx512_core(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
-#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_STATIC_TARGET_ARCH == LIBXSMM_MAX_STATIC_TARGET_ARCH) /* eventually no no need for an indirect call */
+# if (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
   internal_spmdm_compute_bfloat16_thread_avx2(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+# else /* pointer based function call */
+  assert(0 != internal_spmdm_compute_bfloat16_thread);
+  internal_spmdm_compute_bfloat16_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
+# endif
 #else /* pointer based function call */
   assert(0 != internal_spmdm_compute_bfloat16_thread);
   internal_spmdm_compute_bfloat16_thread(handle, transA, transB, alpha, A_sparse, B, transC, beta, C, block_id, tid, nthreads);
@@ -568,6 +528,19 @@ void libxsmm_spmdm_compute_bfloat16_thread(
 }
 
 
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_spmdm_init_check(int archid)
+{
+  if (archid < libxsmm_target_archid
+    && 0 != libxsmm_verbosity) /* library code is expected to be mute */
+  {
+    static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
+      fprintf(stderr, "LIBXSMM: missed to enter \"%s\" code path due to the compiler used!\n", libxsmm_get_target_arch());
+    }
+  }
+}
+
+
 LIBXSMM_API_DEFINITION void libxsmm_spmdm_init(int M, int N, int K, int max_threads,
   libxsmm_spmdm_handle* handle, libxsmm_CSR_sparseslice** libxsmm_output_csr)
 {
@@ -579,21 +552,30 @@ LIBXSMM_API_DEFINITION void libxsmm_spmdm_init(int M, int N, int K, int max_thre
   handle->k  = K;
 
   handle->bm = (M >= 4096 || M <= 1024) ? 512 : 256;
-  if (LIBXSMM_X86_AVX512_CORE <= libxsmm_target_archid) {
+#if defined(LIBXSMM_SPMDM_AVX512_CORE)
+  if (LIBXSMM_X86_AVX512_CORE <= libxsmm_target_archid || LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH) {
+    internal_spmdm_init_check(LIBXSMM_X86_AVX512_CORE);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_avx512_core;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_avx512_core;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_avx512_core;
     internal_spmdm_compute_bfloat16_thread = internal_spmdm_compute_bfloat16_thread_avx512_core;
     handle->bn = 96;
   }
-  else if (LIBXSMM_X86_AVX2 <= libxsmm_target_archid) {
+  else
+#endif
+#if defined(LIBXSMM_SPMDM_AVX2)
+  if (LIBXSMM_X86_AVX2 <= libxsmm_target_archid || LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH) {
+    internal_spmdm_init_check(LIBXSMM_X86_AVX512_MIC);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_avx2;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_avx2;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_avx2;
     internal_spmdm_compute_bfloat16_thread = internal_spmdm_compute_bfloat16_thread_avx2;
     handle->bn = 48;
   }
-  else {
+  else
+#endif
+  {
+    internal_spmdm_init_check(LIBXSMM_X86_AVX);
     internal_spmdm_createSparseSlice_fp32_thread = internal_spmdm_createSparseSlice_fp32_thread_sw;
     internal_spmdm_createSparseSlice_bfloat16_thread = internal_spmdm_createSparseSlice_bfloat16_thread_sw;
     internal_spmdm_compute_fp32_thread = internal_spmdm_compute_fp32_thread_sw;
@@ -610,9 +592,13 @@ LIBXSMM_API_DEFINITION void libxsmm_spmdm_init(int M, int N, int K, int max_thre
   internal_spmdm_allocate_scratch(handle, max_threads);
 
   /* Initialize shuffle masks for the computation */
+#if defined(LIBXSMM_SPMDM_AVX)
   if (LIBXSMM_X86_AVX <= libxsmm_target_archid) {
     internal_spmdm_init_shufmask_avx();
   }
+  assert(0 != internal_spmdm_shufmasks_32);
+  assert(0 != internal_spmdm_shufmasks_16);
+#endif
 
   /* post-conditions */
   assert(0 != internal_spmdm_createSparseSlice_fp32_thread);
