@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2016, Intel Corporation                                **
+** Copyright (c) 2009-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -29,13 +29,14 @@
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include <libxsmm_timer.h>
+#include <libxsmm_intrinsics_x86.h>
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
 #if defined(_WIN32)
 # include <Windows.h>
-#elif defined(__GNUC__)
+#elif defined(__GNUC__) || defined(__PGI) || defined(_CRAYC)
 # include <sys/time.h>
 # include <time.h>
 #endif
@@ -43,8 +44,18 @@
 # pragma offload_attribute(pop)
 #endif
 
+#if (defined(__GNUC__) || defined(__INTEL_COMPILER)) && \
+  ((4294967295U < (__SIZE_MAX__)) || defined(_WIN64) || defined(_CRAYC))
+# define LIBXSMM_TIMER_RDTSC(CYCLE) { unsigned long long libxsmm_timer_rdtsc_hi_; \
+    __asm__ __volatile__ ("rdtsc" : "=a"(CYCLE), "=d"(libxsmm_timer_rdtsc_hi_)); \
+    CYCLE |= libxsmm_timer_rdtsc_hi_ << 32; \
+  }
+#elif defined(_rdtsc) || defined(_WIN32)
+# define LIBXSMM_TIMER_RDTSC(CYCLE) (CYCLE = __rdtsc())
+#endif
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE unsigned long long libxsmm_timer_tick(void)
+
+LIBXSMM_API_DEFINITION unsigned long long libxsmm_timer_tick(void)
 {
 #if defined(_WIN32)
   LARGE_INTEGER t;
@@ -62,7 +73,23 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE unsigned long long libxsmm_timer_tick(void
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE double libxsmm_timer_duration(unsigned long long tick0, unsigned long long tick1)
+LIBXSMM_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_GENERIC)
+unsigned long long libxsmm_timer_xtick(void)
+{
+  unsigned long long result;
+#if defined(LIBXSMM_TIMER_RDTSC)
+  LIBXSMM_TIMER_RDTSC(result);
+#else
+  LIBXSMM_MESSAGE("================================================================================")
+  LIBXSMM_MESSAGE("LIBXSMM: Support for the RDTSC intrinsic appears to be unavailable!")
+  LIBXSMM_MESSAGE("================================================================================")
+  result = libxsmm_timer_tick();
+#endif
+  return result;
+}
+
+
+LIBXSMM_API_DEFINITION double libxsmm_timer_duration(unsigned long long tick0, unsigned long long tick1)
 {
   const double d = (double)(LIBXSMM_MAX(tick1, tick0) - tick0);
 #if defined(_WIN32)
@@ -75,3 +102,4 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE double libxsmm_timer_duration(unsigned lon
   return d * 1E-6;
 #endif
 }
+
