@@ -77,10 +77,10 @@ libxsmm_convfunction jitted_conv_wu_transpose_pf = (libxsmm_convfunction)handle-
 libxsmm_convfunction jitted_conv_wu_transpose_nooutput_pf = (libxsmm_convfunction)handle->code_upd[5].xconv.sconv;
 #endif
 
-element_output_type *const out = ((element_output_type*)handle->output->data) + (handle->desc.pad_h_out * handle->ofwp + handle->desc.pad_w_out) * handle->ofmblock;
+element_output_type *const out = ((element_output_type*)handle->grad_output->data) + (handle->desc.pad_h_out * handle->ofwp + handle->desc.pad_w_out) * handle->ofmblock;
 LIBXSMM_VLA_DECL(5, element_output_type, output, out, handle->blocksofm, handle->ofhp, handle->ofwp, handle->ofmblock);
-LIBXSMM_VLA_DECL(5, element_input_type, input, (element_input_type*)handle->input->data, handle->blocksifm, handle->ifhp, handle->ifwp, handle->ifmblock);
-LIBXSMM_VLA_DECL(6, element_filter_type, weight, (element_filter_type*)handle->filter->data, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
+LIBXSMM_VLA_DECL(5, element_input_type, input, (element_input_type*)handle->reg_input->data, handle->blocksifm, handle->ifhp, handle->ifwp, handle->ifmblock);
+LIBXSMM_VLA_DECL(6, element_filter_type, weight, (element_filter_type*)handle->grad_filter->data, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
 #ifdef LIBXSMM_WU_TRANSPOSE_OFW_IFM
 LIBXSMM_VLA_DECL(5, element_input_type, tr_input, (element_input_type*)handle->scratch3, handle->blocksifm, handle->ifhp, handle->ifmblock, handle->ifwp);
 #endif
@@ -94,7 +94,7 @@ unsigned int stride_h = handle->desc.u;
 
 #ifdef LIBXSMM_WU_PER_THREAD_ALLOCATION
 element_filter_type* remote_weight_ptr = 0;
-element_filter_type* weight_ptr = (element_filter_type*)handle->filter->data;
+element_filter_type* weight_ptr = (element_filter_type*)handle->grad_filter->data;
 element_filter_type* per_thread_weight_ptr = ((element_filter_type*)handle->scratch4) + (ltid*handle->blocksofm*handle->blocksifm*handle->desc.R*handle->desc.S*handle->ifmblock*handle->ofmblock);
 LIBXSMM_VLA_DECL(6, element_filter_type, per_thread_weight, per_thread_weight_ptr, handle->blocksifm, handle->desc.R, handle->desc.S, handle->ifmblock, handle->ofmblock);
 /* number of tasks that could be run in parallel */
@@ -277,7 +277,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
     }
     /* lazy barrier init */
     if (handle->upd_use_external_reduce == 0) {
-      libxsmm_barrier_init((libxsmm_barrier*)handle->scratch2, ltid);
+      libxsmm_barrier_init(handle->barrier, ltid);
     }
 #endif
 
@@ -380,7 +380,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
 #ifdef LIBXSMM_WU_PER_THREAD_ALLOCATION
         /* perform reduction */
         if (handle->upd_use_external_reduce == 0) {
-          libxsmm_barrier_wait((libxsmm_barrier*)handle->scratch2, ltid);
+          libxsmm_barrier_wait(handle->barrier, ltid);
           for ( i = 0; i < handle->desc.threads; i++ ) {
             remote_weight_ptr = ((element_filter_type*)handle->scratch4) + (i*reduce_work);
             for ( j = reduce_thr_begin; j < reduce_thr_end; j++) {
@@ -393,7 +393,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
 
 #ifdef LIBXSMM_WU_TRANSPOSE_OFW_IFM
         /* lazy barrier init */
-        libxsmm_barrier_init((libxsmm_barrier*)handle->scratch2, ltid);
+        libxsmm_barrier_init(handle->barrier, ltid);
         /* First transpose IFW and IFM */
         for (imgifm1 = transpose_thr_begin; imgifm1 < transpose_thr_end; ++imgifm1) {
           img = imgifm1/handle->blocksifm;
@@ -407,7 +407,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
             }
           }
         }
-        libxsmm_barrier_wait((libxsmm_barrier*)handle->scratch2, ltid);
+        libxsmm_barrier_wait(handle->barrier, ltid);
 
         for (ofm1ifm1 = thr_begin; ofm1ifm1 < thr_end; ++ofm1ifm1) {
           ofm1 = ofm1ifm1/handle->blocksifm;
@@ -524,7 +524,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
 #ifdef LIBXSMM_WU_PER_THREAD_ALLOCATION
           /* lazy barrier init */
           if (handle->upd_use_external_reduce == 0) {
-            libxsmm_barrier_init((libxsmm_barrier*)handle->scratch2, ltid);
+            libxsmm_barrier_init(handle->barrier, ltid);
           }
 #endif
 #ifdef LIBXSMM_WU_PER_THREAD_ALLOCATION
@@ -648,7 +648,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
             /* perform reduction */
             /* TODO COMPLETE THIS USING ATOMIC INCREMENTS PLEASE */
             if (handle->upd_use_external_reduce == 0) {
-              libxsmm_barrier_wait((libxsmm_barrier*)handle->scratch2, ltid);
+              libxsmm_barrier_wait(handle->barrier, ltid);
               for ( i = 0; i < handle->desc.threads; i++ ) {
                 remote_weight_ptr = ((element_filter_type*)handle->scratch4) + (i*reduce_work);
                 for ( j = reduce_thr_begin; j < reduce_thr_end; j++) {
@@ -776,7 +776,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
   }
   /* lazy barrier init */
   if (handle->upd_use_external_reduce == 0) {
-    libxsmm_barrier_init((libxsmm_barrier*)handle->scratch2, ltid);
+    libxsmm_barrier_init(handle->barrier, ltid);
   }
   for (ofm1ifm1img = img_parallel_thr_begin; ofm1ifm1img < img_parallel_thr_end; ++ofm1ifm1img) {
     img = ofm1ifm1img / (handle->blocksifm * handle->blocksofm);
@@ -793,7 +793,7 @@ if ( libxsmm_get_target_archid() == LIBXSMM_X86_AVX512_MIC ||
     }
   }
   if (handle->upd_use_external_reduce == 0) {
-    libxsmm_barrier_wait((libxsmm_barrier*)handle->scratch2, ltid);
+    libxsmm_barrier_wait(handle->barrier, ltid);
     /* reduce weights */
     for ( i = 0; i < handle->desc.threads; i++ ) {
       remote_weight_ptr = ((element_filter_type*)handle->scratch4) + (i*reduce_work);
